'use strict';

var React = require('react');
var useSafeLayoutEffect = require('../useSafeLayoutEffect-cjs.js');
var FormField = require('../FormField-cjs.js');
require('classnames');
var tslib_es6 = require('../tslib.es6-cjs.js');
require('react-hook-form');
require('framer-motion');
require('@jobber/design');
require('../Button-cjs.js');
var omit = require('../omit-cjs.js');
require('../Icon-cjs.js');
require('../Text-cjs.js');
require('../Typography-cjs.js');
require('../useFormFieldFocus-cjs.js');
require('../InputValidation-cjs.js');
require('../Spinner-cjs.js');
require('react-router-dom');
require('../_commonjsHelpers-cjs.js');
require('../_baseGet-cjs.js');
require('../isTypedArray-cjs.js');
require('../isObjectLike-cjs.js');
require('../identity-cjs.js');
require('../_getTag-cjs.js');
require('../isSymbol-cjs.js');
require('../keysIn-cjs.js');
require('../_baseAssignValue-cjs.js');
require('../_baseFlatten-cjs.js');
require('../_setToString-cjs.js');

/**
 * Hook for resizing a textarea based on its content. The textarea will grow up to the max number of rows specified.
 */
function useTextAreaResize({ rows, value, inputRef, wrapperRef, }) {
    const rowRange = getRowRange(rows);
    useSafeLayoutEffect.useSafeLayoutEffect_1(() => {
        if (inputRef &&
            inputRef.current instanceof HTMLTextAreaElement &&
            wrapperRef &&
            wrapperRef.current instanceof HTMLDivElement) {
            resize();
        }
    }, [inputRef.current, wrapperRef.current]);
    // When the consumer passes a new controlled value, we need to recheck the size.
    // The timeout ensures the DOM has a enough time to render the new text before
    // we access the height.
    useSafeLayoutEffect.useSafeLayoutEffect_1(() => {
        setTimeout(() => {
            if (inputRef &&
                inputRef.current instanceof HTMLTextAreaElement &&
                wrapperRef &&
                wrapperRef.current instanceof HTMLDivElement) {
                resize();
            }
        }, 0);
    }, [value]);
    function resize() {
        if (inputRef &&
            inputRef.current instanceof HTMLTextAreaElement &&
            wrapperRef &&
            (wrapperRef === null || wrapperRef === void 0 ? void 0 : wrapperRef.current) instanceof HTMLDivElement) {
            if (rowRange.min === rowRange.max)
                return;
            inputRef.current.style.flexBasis = "auto";
            wrapperRef.current.style.height = "auto";
            inputRef.current.style.flexBasis =
                textAreaHeight(inputRef.current) + "px";
        }
    }
    function textAreaHeight(textArea) {
        const { lineHeight, borderBottomWidth, borderTopWidth, paddingBottom, paddingTop, } = window.getComputedStyle(textArea);
        const maxHeight = rowRange.max * parseFloat(lineHeight) +
            parseFloat(borderTopWidth) +
            parseFloat(borderBottomWidth) +
            parseFloat(paddingTop) +
            parseFloat(paddingBottom);
        const scrollHeight = textArea.scrollHeight +
            parseFloat(borderTopWidth) +
            parseFloat(borderBottomWidth);
        return Math.min(scrollHeight, maxHeight);
    }
    return { resize, rowRange };
}
function getRowRange(rows) {
    if (rows === undefined) {
        return { min: 3, max: 3 };
    }
    else if (typeof rows === "object") {
        return { min: rows.min, max: rows.max };
    }
    else {
        return { min: rows, max: rows };
    }
}

function InputTextInternal(props, ref) {
    const inputRef = React.useRef(null);
    const actionsRef = React.useRef(null);
    const wrapperRef = React.useRef(null);
    const { resize, rowRange } = useTextAreaResize({
        rows: props.rows,
        value: props.value,
        inputRef,
        wrapperRef,
    });
    React.useImperativeHandle(ref, () => ({
        insert: (text) => {
            insertText(text);
        },
        blur: () => {
            const input = inputRef.current;
            if (input) {
                input.blur();
            }
        },
        focus: () => {
            const input = inputRef.current;
            if (input) {
                input.focus();
            }
        },
        scrollIntoView: arg => {
            const input = inputRef.current;
            if (input) {
                input.scrollIntoView(arg);
            }
        },
    }));
    return (React.createElement(FormField.FormField, Object.assign({}, props, { type: props.multiline ? "textarea" : "text", inputRef: inputRef, actionsRef: actionsRef, wrapperRef: wrapperRef, onChange: handleChange, rows: rowRange.min })));
    function handleChange(newValue) {
        props.onChange && props.onChange(newValue);
        resize();
    }
    function insertText(text) {
        var _a;
        const input = inputRef.current;
        if (input) {
            insertAtCursor(input, text);
            const newValue = input.value;
            (_a = actionsRef.current) === null || _a === void 0 ? void 0 : _a.setValue(newValue);
            props.onChange && props.onChange(newValue);
        }
    }
}
const InputText$1 = React.forwardRef(InputTextInternal);
function insertAtCursor(input, newText) {
    const start = input.selectionStart || 0;
    const end = input.selectionEnd || 0;
    const text = input.value;
    const before = text.substring(0, start);
    const after = text.substring(end, text.length);
    input.value = before + newText + after;
    input.selectionStart = input.selectionEnd = start + newText.length;
    input.focus();
}

/**
 * Combines the actions on the InputText such as onChange, onEnter, onFocus, onBlur, and onClear to forward information to the consumers of the InputText.
 * DO not repeat this pattern. We are doing this as a proof of concept relating to the refactoring of Form inputs to see what can be removed.
 */
function useInputTextActions({ onChange, inputRef, onEnter, onFocus, onBlur, onKeyDown, }) {
    function handleClear() {
        var _a;
        handleBlur();
        onChange && onChange("");
        (_a = inputRef === null || inputRef === void 0 ? void 0 : inputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }
    function handleChange(event) {
        const newValue = event.currentTarget.value;
        onChange === null || onChange === void 0 ? void 0 : onChange(newValue, event);
    }
    function handleKeyDown(event) {
        onKeyDown === null || onKeyDown === void 0 ? void 0 : onKeyDown(event);
        if (!onEnter)
            return;
        if (event.key !== "Enter")
            return;
        if (event.shiftKey || event.ctrlKey)
            return;
        event.preventDefault();
        onEnter && onEnter(event);
    }
    function handleFocus(event) {
        onFocus === null || onFocus === void 0 ? void 0 : onFocus(event);
    }
    function handleBlur(event) {
        onBlur === null || onBlur === void 0 ? void 0 : onBlur(event);
    }
    return {
        handleClear,
        handleChange,
        handleKeyDown,
        handleFocus,
        handleBlur,
    };
}

/**
 * Provides the props for the html fields rendered by the html input.
 *  DO not repeat this pattern. We are doing this as a proof of concept relating to the refactoring of Form inputs to see what can be removed.
 */
function useInputTextFormField(_a) {
    var { id, name, description, inline, handleChange, handleBlur, handleFocus, handleKeyDown, error } = _a, rest = tslib_es6.__rest(_a, ["id", "name", "description", "inline", "handleChange", "handleBlur", "handleFocus", "handleKeyDown", "error"]);
    const descriptionIdentifier = `descriptionUUID--${id}`;
    const fieldProps = Object.assign(Object.assign(Object.assign(Object.assign({}, rest), { id, className: FormField.styles.input, name, disabled: rest.disabled, inputMode: rest.inputMode, onChange: handleChange, onBlur: handleBlur, onFocus: handleFocus }), (description &&
        !inline && { "aria-describedby": descriptionIdentifier })), { "aria-invalid": rest["aria-invalid"] || error || rest.invalid ? true : undefined, autoFocus: rest.autoFocus, invalid: error || rest.invalid ? "true" : undefined, onKeyDown: handleKeyDown });
    return { fieldProps, descriptionIdentifier };
}

const InputTextSPAR = React.forwardRef(function InputTextInternal(props, inputRefs) {
    var _a, _b, _c;
    const inputTextRef = React.useRef(null);
    const wrapperRef = React.useRef(null);
    const legacyPropHelper = Object.assign(Object.assign({}, props), { version: 1 });
    const id = useInputTextId(props);
    const { rowRange } = useTextAreaResize({
        rows: props.rows,
        value: props.value,
        inputRef: inputTextRef,
        wrapperRef: wrapperRef,
    });
    const type = props.multiline ? "textarea" : "text";
    const { inputStyle } = FormField.useFormFieldWrapperStyles(legacyPropHelper);
    const { name } = FormField.useAtlantisFormFieldName({
        nameProp: props.name,
        id: id,
    });
    const { handleChange, handleBlur, handleFocus, handleKeyDown, handleClear } = useInputTextActions({
        onChange: props.onChange,
        onBlur: props.onBlur,
        onFocus: props.onFocus,
        onKeyDown: props.onKeyDown,
        onEnter: props.onEnter,
        inputRef: inputTextRef,
    });
    const inputProps = omit.omit(props, [
        "onChange",
        "onBlur",
        "onFocus",
        "onEnter",
        "size",
        "placeholder",
        "multiline",
        "prefix",
        "suffix",
        "version",
    ]);
    const { fieldProps, descriptionIdentifier } = useInputTextFormField(Object.assign(Object.assign({}, inputProps), { id,
        name,
        handleChange,
        handleBlur,
        handleFocus,
        handleKeyDown }));
    return (React.createElement(FormField.FormFieldWrapper, { disabled: props.disabled, size: props.size, align: props.align, inline: props.inline, autofocus: props.autoFocus, name: name, wrapperRef: wrapperRef, error: (_a = props.error) !== null && _a !== void 0 ? _a : "", invalid: Boolean(props.error || props.invalid), identifier: id, descriptionIdentifier: descriptionIdentifier, description: props.description, clearable: (_b = props.clearable) !== null && _b !== void 0 ? _b : "never", onClear: handleClear, type: props.multiline ? "textarea" : "text", placeholder: props.placeholder, value: props.value, prefix: props.prefix, suffix: props.suffix, rows: rowRange.min, toolbar: props.toolbar, toolbarVisibility: props.toolbarVisibility },
        React.createElement(React.Fragment, null,
            type === "textarea" ? (React.createElement(TextArea, { fieldProps: fieldProps, rowRange: rowRange, inputRefs: [inputRefs, inputTextRef], value: props.value, inputStyle: inputStyle })) : (React.createElement(TextInput, { fieldProps: fieldProps, inputRefs: [inputRefs, inputTextRef], value: props.value, inputStyle: inputStyle })),
            React.createElement(FormField.FormFieldPostFix, { variation: "spinner", visible: (_c = props.loading) !== null && _c !== void 0 ? _c : false }),
            props.children)));
});
function useInputTextId(props) {
    const generatedId = React.useId();
    return props.id || generatedId;
}
function TextArea({ fieldProps, rowRange, inputRefs, value, inputStyle, }) {
    return (React.createElement("textarea", Object.assign({}, fieldProps, { rows: rowRange.min, ref: FormField.mergeRefs(inputRefs), className: inputStyle, value: value })));
}
function TextInput({ fieldProps, inputRefs, value, inputStyle, }) {
    return (React.createElement("input", Object.assign({}, fieldProps, { ref: FormField.mergeRefs(inputRefs), className: inputStyle, value: value })));
}

function isNewInputTextProps(props) {
    return props.version === 2;
}
const InputText = React.forwardRef(function InputTextShim(props, ref) {
    if (isNewInputTextProps(props)) {
        return (React.createElement(InputTextSPAR, Object.assign({}, props, { ref: ref })));
    }
    else {
        return (React.createElement(InputText$1, Object.assign({}, props, { ref: ref })));
    }
});

exports.InputText = InputText;
