import React__default, { forwardRef, useRef, useImperativeHandle, useId } from 'react';
import { u as useSafeLayoutEffect_1 } from '../useSafeLayoutEffect-es.js';
import { k as FormField, s as styles, j as useFormFieldWrapperStyles, b as useAtlantisFormFieldName, f as FormFieldWrapper, l as FormFieldPostFix, m as mergeRefs } from '../FormField-es.js';
import 'classnames';
import { _ as __rest } from '../tslib.es6-es.js';
import 'react-hook-form';
import 'framer-motion';
import '@jobber/design';
import '../Button-es.js';
import { o as omit } from '../omit-es.js';
import '../Icon-es.js';
import '../Text-es.js';
import '../Typography-es.js';
import '../useFormFieldFocus-es.js';
import '../InputValidation-es.js';
import '../Spinner-es.js';
import 'react-router-dom';
import '../_commonjsHelpers-es.js';
import '../_baseGet-es.js';
import '../isTypedArray-es.js';
import '../isObjectLike-es.js';
import '../identity-es.js';
import '../_getTag-es.js';
import '../isSymbol-es.js';
import '../keysIn-es.js';
import '../_baseAssignValue-es.js';
import '../_baseFlatten-es.js';
import '../_setToString-es.js';

/**
 * Hook for resizing a textarea based on its content. The textarea will grow up to the max number of rows specified.
 */
function useTextAreaResize({ rows, value, inputRef, wrapperRef, }) {
    const rowRange = getRowRange(rows);
    useSafeLayoutEffect_1(() => {
        if (inputRef &&
            inputRef.current instanceof HTMLTextAreaElement &&
            wrapperRef &&
            wrapperRef.current instanceof HTMLDivElement) {
            resize();
        }
    }, [inputRef.current, wrapperRef.current]);
    // When the consumer passes a new controlled value, we need to recheck the size.
    // The timeout ensures the DOM has a enough time to render the new text before
    // we access the height.
    useSafeLayoutEffect_1(() => {
        setTimeout(() => {
            if (inputRef &&
                inputRef.current instanceof HTMLTextAreaElement &&
                wrapperRef &&
                wrapperRef.current instanceof HTMLDivElement) {
                resize();
            }
        }, 0);
    }, [value]);
    function resize() {
        if (inputRef &&
            inputRef.current instanceof HTMLTextAreaElement &&
            wrapperRef &&
            (wrapperRef === null || wrapperRef === void 0 ? void 0 : wrapperRef.current) instanceof HTMLDivElement) {
            if (rowRange.min === rowRange.max)
                return;
            inputRef.current.style.flexBasis = "auto";
            wrapperRef.current.style.height = "auto";
            inputRef.current.style.flexBasis =
                textAreaHeight(inputRef.current) + "px";
        }
    }
    function textAreaHeight(textArea) {
        const { lineHeight, borderBottomWidth, borderTopWidth, paddingBottom, paddingTop, } = window.getComputedStyle(textArea);
        const maxHeight = rowRange.max * parseFloat(lineHeight) +
            parseFloat(borderTopWidth) +
            parseFloat(borderBottomWidth) +
            parseFloat(paddingTop) +
            parseFloat(paddingBottom);
        const scrollHeight = textArea.scrollHeight +
            parseFloat(borderTopWidth) +
            parseFloat(borderBottomWidth);
        return Math.min(scrollHeight, maxHeight);
    }
    return { resize, rowRange };
}
function getRowRange(rows) {
    if (rows === undefined) {
        return { min: 3, max: 3 };
    }
    else if (typeof rows === "object") {
        return { min: rows.min, max: rows.max };
    }
    else {
        return { min: rows, max: rows };
    }
}

function InputTextInternal(props, ref) {
    const inputRef = useRef(null);
    const actionsRef = useRef(null);
    const wrapperRef = useRef(null);
    const { resize, rowRange } = useTextAreaResize({
        rows: props.rows,
        value: props.value,
        inputRef,
        wrapperRef,
    });
    useImperativeHandle(ref, () => ({
        insert: (text) => {
            insertText(text);
        },
        blur: () => {
            const input = inputRef.current;
            if (input) {
                input.blur();
            }
        },
        focus: () => {
            const input = inputRef.current;
            if (input) {
                input.focus();
            }
        },
        scrollIntoView: arg => {
            const input = inputRef.current;
            if (input) {
                input.scrollIntoView(arg);
            }
        },
    }));
    return (React__default.createElement(FormField, Object.assign({}, props, { type: props.multiline ? "textarea" : "text", inputRef: inputRef, actionsRef: actionsRef, wrapperRef: wrapperRef, onChange: handleChange, rows: rowRange.min })));
    function handleChange(newValue) {
        props.onChange && props.onChange(newValue);
        resize();
    }
    function insertText(text) {
        var _a;
        const input = inputRef.current;
        if (input) {
            insertAtCursor(input, text);
            const newValue = input.value;
            (_a = actionsRef.current) === null || _a === void 0 ? void 0 : _a.setValue(newValue);
            props.onChange && props.onChange(newValue);
        }
    }
}
const InputText$1 = forwardRef(InputTextInternal);
function insertAtCursor(input, newText) {
    const start = input.selectionStart || 0;
    const end = input.selectionEnd || 0;
    const text = input.value;
    const before = text.substring(0, start);
    const after = text.substring(end, text.length);
    input.value = before + newText + after;
    input.selectionStart = input.selectionEnd = start + newText.length;
    input.focus();
}

/**
 * Combines the actions on the InputText such as onChange, onEnter, onFocus, onBlur, and onClear to forward information to the consumers of the InputText.
 * DO not repeat this pattern. We are doing this as a proof of concept relating to the refactoring of Form inputs to see what can be removed.
 */
function useInputTextActions({ onChange, inputRef, onEnter, onFocus, onBlur, onKeyDown, }) {
    function handleClear() {
        var _a;
        handleBlur();
        onChange && onChange("");
        (_a = inputRef === null || inputRef === void 0 ? void 0 : inputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }
    function handleChange(event) {
        const newValue = event.currentTarget.value;
        onChange === null || onChange === void 0 ? void 0 : onChange(newValue, event);
    }
    function handleKeyDown(event) {
        onKeyDown === null || onKeyDown === void 0 ? void 0 : onKeyDown(event);
        if (!onEnter)
            return;
        if (event.key !== "Enter")
            return;
        if (event.shiftKey || event.ctrlKey)
            return;
        event.preventDefault();
        onEnter && onEnter(event);
    }
    function handleFocus(event) {
        onFocus === null || onFocus === void 0 ? void 0 : onFocus(event);
    }
    function handleBlur(event) {
        onBlur === null || onBlur === void 0 ? void 0 : onBlur(event);
    }
    return {
        handleClear,
        handleChange,
        handleKeyDown,
        handleFocus,
        handleBlur,
    };
}

/**
 * Provides the props for the html fields rendered by the html input.
 *  DO not repeat this pattern. We are doing this as a proof of concept relating to the refactoring of Form inputs to see what can be removed.
 */
function useInputTextFormField(_a) {
    var { id, name, description, inline, handleChange, handleBlur, handleFocus, handleKeyDown, error } = _a, rest = __rest(_a, ["id", "name", "description", "inline", "handleChange", "handleBlur", "handleFocus", "handleKeyDown", "error"]);
    const descriptionIdentifier = `descriptionUUID--${id}`;
    const fieldProps = Object.assign(Object.assign(Object.assign(Object.assign({}, rest), { id, className: styles.input, name, disabled: rest.disabled, inputMode: rest.inputMode, onChange: handleChange, onBlur: handleBlur, onFocus: handleFocus }), (description &&
        !inline && { "aria-describedby": descriptionIdentifier })), { "aria-invalid": rest["aria-invalid"] || error || rest.invalid ? true : undefined, autoFocus: rest.autoFocus, invalid: error || rest.invalid ? "true" : undefined, onKeyDown: handleKeyDown });
    return { fieldProps, descriptionIdentifier };
}

const InputTextSPAR = forwardRef(function InputTextInternal(props, inputRefs) {
    var _a, _b, _c;
    const inputTextRef = React__default.useRef(null);
    const wrapperRef = React__default.useRef(null);
    const legacyPropHelper = Object.assign(Object.assign({}, props), { version: 1 });
    const id = useInputTextId(props);
    const { rowRange } = useTextAreaResize({
        rows: props.rows,
        value: props.value,
        inputRef: inputTextRef,
        wrapperRef: wrapperRef,
    });
    const type = props.multiline ? "textarea" : "text";
    const { inputStyle } = useFormFieldWrapperStyles(legacyPropHelper);
    const { name } = useAtlantisFormFieldName({
        nameProp: props.name,
        id: id,
    });
    const { handleChange, handleBlur, handleFocus, handleKeyDown, handleClear } = useInputTextActions({
        onChange: props.onChange,
        onBlur: props.onBlur,
        onFocus: props.onFocus,
        onKeyDown: props.onKeyDown,
        onEnter: props.onEnter,
        inputRef: inputTextRef,
    });
    const inputProps = omit(props, [
        "onChange",
        "onBlur",
        "onFocus",
        "onEnter",
        "size",
        "placeholder",
        "multiline",
        "prefix",
        "suffix",
        "version",
    ]);
    const { fieldProps, descriptionIdentifier } = useInputTextFormField(Object.assign(Object.assign({}, inputProps), { id,
        name,
        handleChange,
        handleBlur,
        handleFocus,
        handleKeyDown }));
    return (React__default.createElement(FormFieldWrapper, { disabled: props.disabled, size: props.size, align: props.align, inline: props.inline, autofocus: props.autoFocus, name: name, wrapperRef: wrapperRef, error: (_a = props.error) !== null && _a !== void 0 ? _a : "", invalid: Boolean(props.error || props.invalid), identifier: id, descriptionIdentifier: descriptionIdentifier, description: props.description, clearable: (_b = props.clearable) !== null && _b !== void 0 ? _b : "never", onClear: handleClear, type: props.multiline ? "textarea" : "text", placeholder: props.placeholder, value: props.value, prefix: props.prefix, suffix: props.suffix, rows: rowRange.min, toolbar: props.toolbar, toolbarVisibility: props.toolbarVisibility },
        React__default.createElement(React__default.Fragment, null,
            type === "textarea" ? (React__default.createElement(TextArea, { fieldProps: fieldProps, rowRange: rowRange, inputRefs: [inputRefs, inputTextRef], value: props.value, inputStyle: inputStyle })) : (React__default.createElement(TextInput, { fieldProps: fieldProps, inputRefs: [inputRefs, inputTextRef], value: props.value, inputStyle: inputStyle })),
            React__default.createElement(FormFieldPostFix, { variation: "spinner", visible: (_c = props.loading) !== null && _c !== void 0 ? _c : false }),
            props.children)));
});
function useInputTextId(props) {
    const generatedId = useId();
    return props.id || generatedId;
}
function TextArea({ fieldProps, rowRange, inputRefs, value, inputStyle, }) {
    return (React__default.createElement("textarea", Object.assign({}, fieldProps, { rows: rowRange.min, ref: mergeRefs(inputRefs), className: inputStyle, value: value })));
}
function TextInput({ fieldProps, inputRefs, value, inputStyle, }) {
    return (React__default.createElement("input", Object.assign({}, fieldProps, { ref: mergeRefs(inputRefs), className: inputStyle, value: value })));
}

function isNewInputTextProps(props) {
    return props.version === 2;
}
const InputText = forwardRef(function InputTextShim(props, ref) {
    if (isNewInputTextProps(props)) {
        return (React__default.createElement(InputTextSPAR, Object.assign({}, props, { ref: ref })));
    }
    else {
        return (React__default.createElement(InputText$1, Object.assign({}, props, { ref: ref })));
    }
});

export { InputText };
