import React__default, { useState, useCallback, useEffect, useRef, useId } from 'react';
import { _ as __rest } from '../tslib.es6-es.js';
import { d as debounce } from '../debounce-es.js';
import { j as useFormFieldWrapperStyles, f as FormFieldWrapper, k as FormField } from '../FormField-es.js';
import '@jobber/design';
import 'classnames';
import 'react-hook-form';
import '../Button-es.js';
import { o as omit } from '../omit-es.js';
import 'framer-motion';
import '../_commonjsHelpers-es.js';
import '../isObjectLike-es.js';
import '../isSymbol-es.js';
import '../Icon-es.js';
import '../Text-es.js';
import '../Typography-es.js';
import '../useFormFieldFocus-es.js';
import '../InputValidation-es.js';
import '../Spinner-es.js';
import 'react-router-dom';
import '../_baseGet-es.js';
import '../isTypedArray-es.js';
import '../identity-es.js';
import '../_getTag-es.js';
import '../keysIn-es.js';
import '../_baseAssignValue-es.js';
import '../_baseFlatten-es.js';
import '../_setToString-es.js';

const DEBOUNCE_TIME = 300;
function useTimePredict({ value, handleChange }) {
    const [IS_12_HOUR_FORMAT, set12HourFormat] = useState(false);
    const [typedTime, setTypedTime] = useState("");
    const predictTime = useCallback(debounce(() => {
        if (value)
            return;
        handleChange(predictHours(typedTime, IS_12_HOUR_FORMAT));
    }, DEBOUNCE_TIME), [typedTime, value, handleChange, IS_12_HOUR_FORMAT]);
    useEffect(() => {
        set12HourFormat(Boolean(Intl.DateTimeFormat(navigator.language, {
            hour: "numeric",
        }).resolvedOptions().hour12));
    }, []);
    /**
     * Predict the hour when user types a number
     */
    useEffect(() => {
        /**
         * Don't try to predict if the user types 0 as the user would almost always
         * type another number after 0.
         */
        if (typedTime && typedTime !== "0") {
            predictTime();
            if ((IS_12_HOUR_FORMAT && typedTime !== "1") || typedTime.length === 2) {
                // Immediately predict the hour
                predictTime.flush();
            }
        }
        return predictTime.cancel;
    }, [typedTime]);
    /**
     * Reset typed time when the value changed
     */
    useEffect(() => {
        setTypedTime("");
    }, [value]);
    return {
        setTypedTime,
    };
}
function predictHours(time, is12HourFormat = false) {
    const today = new Date();
    const currentHour = today.getHours();
    const parsedTime = parseInt(time, 10);
    let predictedTime;
    if (is12HourFormat) {
        predictedTime = predict12Hours(parsedTime, currentHour);
    }
    else {
        predictedTime = predict24Hours(time, parsedTime, currentHour);
    }
    return formatHour(predictedTime);
}
function predict12Hours(parsedTime, currentHour) {
    /**
     * We need to predict what the user wants when they type 1 since it could be
     * 10, 11, 12, or 1.
     *
     * If the current hour is over 12PM, we can skip this and go to the next logic.
     */
    if (parsedTime === 1 && currentHour < 12) {
        if (currentHour < 10) {
            return 10;
        }
        else {
            return currentHour + 1;
        }
    }
    /**
     * Typing 1-5 predicts that the user want that exact hour on the afternoon.
     */
    if (parsedTime <= 5) {
        return parsedTime + 12;
    }
    /**
     * Typing 13-15 predicts that the user want the 2nd number as the minute interval.
     */
    if (parsedTime > 12 && parsedTime <= 15) {
        const timeArray = parsedTime.toString().split("").map(Number); // 13 -> [1, 3]
        const convertToHoursAndMinutes = timeArray.map((time, i) => {
            if (i === 0)
                return time + 12; // 1 -> 13 as 1PM
            if (i === 1)
                return time * 10; // 3 -> 30 as 30 minutes
        });
        return Number(convertToHoursAndMinutes.join("")); // [13, 30] -> 1330
    }
    /**
     * Anything after 5 will be predicted to be set as AM.
     */
    return parsedTime;
}
function predict24Hours(time, parsedTime, currentHour) {
    /**
     * Typing 1 could be predicted from 10 to 19.
     * Typing 01 skips this logic and sets the time to 01:00.
     */
    if (parsedTime === 1 && time !== "01") {
        if (currentHour < 10 || currentHour > 19) {
            return 10;
        }
        else {
            return currentHour + 1;
        }
    }
    /**
     * Typing 2 could be predicted from 20 to 24.
     * Typing 02 skips this logic and sets the time to 02:00.
     */
    if (parsedTime === 2 && time !== "02") {
        if (currentHour < 20) {
            return 20;
        }
        else {
            return currentHour + 1;
        }
    }
    /**
     * Typing 24 or 00 is set to midnight. This ensures the time doesn't get set
     * to 24 as that would be invalid.
     */
    if (parsedTime === 24 || time === "00") {
        return 0;
    }
    return parsedTime;
}
function formatHour(time) {
    if (time > 99) {
        const splitTime = time.toString().split("");
        if (splitTime.length === 3)
            splitTime.unshift("0"); // 130 -> 0130
        splitTime.splice(2, 0, ":"); // 0130 -> 01:30
        return splitTime.join("");
    }
    if (time < 10) {
        return `0${time}:00`;
    }
    return `${time}:00`;
}

function dateToTimeString(date) {
    if (!(date instanceof Date)) {
        return "";
    }
    // Extract hours and minutes from the Date object
    const hours = date.getHours().toString().padStart(2, "0");
    const minutes = date.getMinutes().toString().padStart(2, "0");
    // Return the time string in HH:MM format
    return `${hours}:${minutes}`;
}
function timeStringToDate(timeString) {
    try {
        const [hours, minutes] = timeString.split(":").map(Number);
        if (isNaN(hours) ||
            isNaN(minutes) ||
            hours < 0 ||
            hours > 24 ||
            minutes < 0 ||
            minutes > 60) {
            return undefined;
        }
        const date = new Date();
        date.setHours(hours, minutes, 0, 0);
        return date;
    }
    catch (_a) {
        return undefined;
    }
}

function InputTimeRebuilt(_a) {
    var _b, _c;
    var { value, onChange } = _a, params = __rest(_a, ["value", "onChange"]);
    const ref = (_b = params.inputRef) !== null && _b !== void 0 ? _b : useRef(null);
    const { setTypedTime } = useTimePredict({
        value,
        handleChange,
    });
    const { inputStyle } = useFormFieldWrapperStyles(params);
    const wrapperRef = React__default.useRef(null);
    const id = getId(params);
    return (React__default.createElement(FormFieldWrapper, { disabled: params.disabled, size: params.size, align: params.align, inline: params.inline, name: params.name, error: params.error || "", identifier: id, descriptionIdentifier: `descriptionUUID--${id}`, invalid: Boolean(params.invalid), description: params.description, clearable: (_c = params.clearable) !== null && _c !== void 0 ? _c : "never", onClear: handleClear, type: "time", readonly: params.readonly, placeholder: params.placeholder, value: dateToTimeString(value), wrapperRef: wrapperRef },
        React__default.createElement("input", { ref: ref, type: "time", name: params.name, className: inputStyle, onBlur: handleBlur, id: id, disabled: params.disabled, readOnly: params.readonly, onChange: handleChangeEvent, onFocus: handleFocus, "data-testid": "ATL-InputTime-input", onKeyUp: e => {
                if (params.disabled || params.readonly)
                    return;
                !isNaN(parseInt(e.key, 10)) && setTypedTime(prev => prev + e.key);
            }, value: dateToTimeString(value) })));
    function handleChangeEvent(event) {
        handleChange(event.target.value);
    }
    function handleChange(newValue) {
        onChange === null || onChange === void 0 ? void 0 : onChange(timeStringToDate(newValue));
    }
    function handleBlur(event) {
        var _a;
        (_a = params.onBlur) === null || _a === void 0 ? void 0 : _a.call(params, event);
        if (ref.current) {
            if (!ref.current.checkValidity()) {
                ref.current.value = "";
            }
        }
    }
    function handleClear() {
        var _a;
        handleBlur();
        onChange === null || onChange === void 0 ? void 0 : onChange(undefined);
        (_a = ref.current) === null || _a === void 0 ? void 0 : _a.focus();
    }
    function handleFocus(event) {
        var _a;
        (_a = params.onFocus) === null || _a === void 0 ? void 0 : _a.call(params, event);
    }
    function getId(props) {
        const generatedId = useId();
        return props.id || generatedId;
    }
}

function InputTime$1(_a) {
    var { defaultValue, value, onChange } = _a, params = __rest(_a, ["defaultValue", "value", "onChange"]);
    const ref = useRef(null);
    const wrapperRef = useRef(null);
    const { setTypedTime } = useTimePredict({ value, handleChange });
    const fieldProps = omit(Object.assign(Object.assign(Object.assign({ onChange: handleChange }, (defaultValue && { defaultValue: dateToTimeString(defaultValue) })), (!defaultValue && { value: dateToTimeString(value) })), params), ["version"]);
    return (React__default.createElement(FormField, Object.assign({ inputRef: ref, type: "time" }, fieldProps, { onBlur: handleBlur, onKeyUp: e => {
            var _a;
            (_a = fieldProps.onKeyUp) === null || _a === void 0 ? void 0 : _a.call(fieldProps, e);
            !isNaN(parseInt(e.key, 10)) && setTypedTime(prev => prev + e.key);
        }, wrapperRef: wrapperRef })));
    function handleChange(newValue) {
        onChange === null || onChange === void 0 ? void 0 : onChange(timeStringToDate(newValue));
    }
    function handleBlur() {
        var _a;
        (_a = params.onBlur) === null || _a === void 0 ? void 0 : _a.call(params);
        // Time inputs doesn't clear the typed value when it's invalid. This should
        // force it to reset the input value when the typed value is invalid.
        if (ref.current) {
            if (!ref.current.checkValidity()) {
                ref.current.value = "";
            }
        }
    }
}

function isNewInputTimeProps(props) {
    return props.version === 2;
}
function InputTime(props) {
    if (isNewInputTimeProps(props)) {
        return React__default.createElement(InputTimeRebuilt, Object.assign({}, props));
    }
    else {
        return React__default.createElement(InputTime$1, Object.assign({}, props));
    }
}

export { InputTime };
