'use strict';

var React = require('react');
var classnames = require('classnames');
var throttle = require('./throttle-cjs.js');
var Typography = require('./Typography-cjs.js');

var styles = {"tabs":"cHJe2HsBNic-","overflow":"eQt1Jis30lo-","overflowRight":"_6Vf7pQfyKKk-","overflowLeft":"jyevrJK7Bcg-","tabRow":"LfIIg9bwrhQ-","tab":"DEdzikxlAkw-","selected":"VDB94VYoyB0-","tabContent":"htdKg6As4bY-","spinning":"XFrAkUebMjI-"};

function useTabsOverflow() {
    const [overflowRight, setOverflowRight] = React.useState(false);
    const [overflowLeft, setOverflowLeft] = React.useState(false);
    const tabRow = React.useRef();
    const handleOverflowing = () => {
        if (tabRow.current) {
            const scrollWidth = tabRow.current.scrollWidth;
            const clientWidth = tabRow.current.clientWidth;
            const maxScroll = scrollWidth - clientWidth;
            const scrollPos = tabRow.current.scrollLeft;
            setOverflowRight(scrollPos >= 0 && scrollPos != maxScroll);
            setOverflowLeft(scrollPos > 0 && scrollPos < scrollWidth);
        }
    };
    const throttledHandleOverflowing = React.useCallback(throttle.throttle(handleOverflowing, 100), []);
    React.useEffect(() => {
        var _a;
        handleOverflowing();
        (_a = tabRow === null || tabRow === void 0 ? void 0 : tabRow.current) === null || _a === void 0 ? void 0 : _a.addEventListener("scroll", throttledHandleOverflowing);
        return () => {
            var _a;
            (_a = tabRow === null || tabRow === void 0 ? void 0 : tabRow.current) === null || _a === void 0 ? void 0 : _a.removeEventListener("scroll", throttledHandleOverflowing);
        };
    });
    return {
        overflowRight,
        overflowLeft,
        tabRow,
    };
}

const useArrowKeyNavigation = ({ elementsRef, onActivate, }) => {
    const handleKeyDown = React.useCallback((event) => {
        const elements = elementsRef.current;
        if (!elements)
            return;
        const currentIndex = getActiveTabIndex(elements);
        if (currentIndex === -1)
            return;
        const focusAndActivateTab = (index) => {
            const element = elements.get(index);
            if (element) {
                element.focus();
                onActivate(index);
            }
        };
        if (event.key === "ArrowRight") {
            event.preventDefault();
            const nextIndex = (currentIndex + 1) % elements.size;
            focusAndActivateTab(nextIndex);
        }
        else if (event.key === "ArrowLeft") {
            event.preventDefault();
            const prevIndex = (currentIndex - 1 + elements.size) % elements.size;
            focusAndActivateTab(prevIndex);
        }
    }, [elementsRef, onActivate]);
    return handleKeyDown;
};
function getActiveTabIndex(elements) {
    const currentTab = Array.from(elements).find(([, element]) => element === document.activeElement);
    if (!currentTab) {
        return -1;
    }
    const [currentIndex] = currentTab;
    return currentIndex;
}

function Tabs({ children, defaultTab = 0, activeTab: controlledActiveTab, onTabChange, }) {
    var _a;
    const tabChildren = getActiveTabs(children);
    const activeTabInitialValue = defaultTab < tabChildren.length ? defaultTab : 0;
    const [internalActiveTab, setInternalActiveTab] = React.useState(activeTabInitialValue);
    const activeTab = controlledActiveTab !== undefined ? controlledActiveTab : internalActiveTab;
    const { overflowRight, overflowLeft, tabRow } = useTabsOverflow();
    const overflowClassNames = classnames(styles.overflow, {
        [styles.overflowRight]: overflowRight,
        [styles.overflowLeft]: overflowLeft,
    });
    const tabRefs = React.useRef(new Map());
    const activateTab = (index) => {
        return () => {
            if (controlledActiveTab === undefined) {
                setInternalActiveTab(index);
            }
            if (onTabChange) {
                onTabChange(index);
            }
        };
    };
    const handleKeyDown = useArrowKeyNavigation({
        elementsRef: tabRefs,
        onActivate: index => activateTab(index)(),
    });
    const activeTabProps = (_a = tabChildren[activeTab]) === null || _a === void 0 ? void 0 : _a.props;
    React.useEffect(() => {
        if (activeTab > tabChildren.length - 1) {
            setInternalActiveTab(activeTabInitialValue);
        }
    }, [tabChildren.length]);
    return (React.createElement("div", { className: styles.tabs },
        React.createElement("div", { className: overflowClassNames },
            React.createElement("ul", { role: "tablist", className: styles.tabRow, ref: tabRow, onKeyDown: handleKeyDown }, React.Children.map(children, child => {
                if (!isChildTab(child)) {
                    return child;
                }
                const index = tabChildren.findIndex(tab => tab.props.label === child.props.label);
                return (React.createElement(InternalTab, { label: child.props.label, selected: activeTab === index, activateTab: activateTab(index), onClick: child.props.onClick, ref: el => {
                        if (el) {
                            tabRefs.current.set(index, el);
                        }
                        else {
                            tabRefs.current.delete(index);
                        }
                    }, tabIndex: activeTab === index ? 0 : -1 }));
            }))),
        React.createElement("section", { role: "tabpanel", className: styles.tabContent, "aria-label": activeTabProps === null || activeTabProps === void 0 ? void 0 : activeTabProps.label }, activeTabProps === null || activeTabProps === void 0 ? void 0 : activeTabProps.children)));
}
function Tab({ label }) {
    return React.createElement(React.Fragment, null, label);
}
const InternalTab = React.forwardRef(({ label, selected, activateTab, onClick, tabIndex }, ref) => {
    const className = classnames(styles.tab, { [styles.selected]: selected });
    return (React.createElement("li", { role: "presentation" },
        React.createElement("button", { type: "button", role: "tab", className: className, onClick: event => {
                activateTab();
                onClick === null || onClick === void 0 ? void 0 : onClick(event);
            }, ref: ref, tabIndex: tabIndex }, typeof label === "string" ? (React.createElement(Typography.Typography, { element: "span", size: "large", fontWeight: "semiBold" }, label)) : (label))));
});
InternalTab.displayName = "InternalTab";
function getActiveTabs(children) {
    const activeTabChildren = [];
    React.Children.toArray(children).forEach(child => {
        if (isChildTab(child)) {
            activeTabChildren.push(child);
        }
    });
    return activeTabChildren;
}
function isChildTab(child) {
    return React.isValidElement(child) && child.type === Tab;
}

exports.Tab = Tab;
exports.Tabs = Tabs;
